/// <reference types="node" />
import type { RouteOptions } from 'fastify';
export declare const ERROR_LABEL = "fastify-autoroutes";
export declare type ValidMethods = 'DELETE' | 'GET' | 'HEAD' | 'PATCH' | 'POST' | 'PUT' | 'OPTIONS';
export declare type AnyRoute = Omit<RouteOptions, 'method' | 'url'>;
export declare type DeleteRoute = AnyRoute;
export declare type GetRoute = Omit<AnyRoute, 'body'>;
export declare type HeadRoute = AnyRoute;
export declare type PatchRoute = AnyRoute;
export declare type PostRoute = AnyRoute;
export declare type PutRoute = AnyRoute;
export declare type OptionsRoute = AnyRoute;
interface Security {
    [key: string]: string[];
}
interface StrictAnyRoute extends AnyRoute {
    schema: {
        summary?: string;
        description?: string;
        security?: Security[];
        tags?: string[];
        consumes?: string[];
        produces?: string[];
        body?: any;
        querystring?: any;
        params?: any;
        headers?: any;
        response?: {
            [key: number]: any;
        };
    };
}
export declare type StrictDeleteRoute = StrictAnyRoute;
export declare type StrictGetRoute = Omit<StrictAnyRoute, 'body'>;
export declare type StrictHeadRoute = StrictAnyRoute;
export declare type StrictPatchRoute = StrictAnyRoute;
export declare type StrictPostRoute = StrictAnyRoute;
export declare type StrictPutRoute = StrictAnyRoute;
export declare type StrictOptionsRoute = StrictAnyRoute;
export interface Resource {
    delete?: DeleteRoute;
    get?: GetRoute;
    head?: HeadRoute;
    patch?: PatchRoute;
    post?: PostRoute;
    put?: PutRoute;
    options?: OptionsRoute;
}
export interface StrictResource {
    delete?: StrictDeleteRoute;
    get?: StrictGetRoute;
    head?: StrictHeadRoute;
    patch?: StrictPatchRoute;
    post?: StrictPostRoute;
    put?: StrictPutRoute;
    options?: StrictOptionsRoute;
}
interface FastifyAutoroutesOptions {
    dir?: string;
    prefix?: string;
}
declare const _default: import("fastify").FastifyPluginCallback<FastifyAutoroutesOptions, import("http").Server>;
export default _default;
//# sourceMappingURL=index.d.ts.map